package com.nhnst.oc.sso.controller;

import java.io.IOException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import okhttp3.Call;
import okhttp3.FormBody;
import okhttp3.Request;
import okhttp3.Response;

@Controller
public class AdditionApiController extends AbstractLoginController {
	private static final String API_KEY = "ed063112e0f6425ea483c7a8d7a92b28";
	protected final Log logger = LogFactory.getLog(AdditionApiController.class);

	/**
	 * Callback AdditionApi sample
	 * 
	 * @param request
	 * @param response
	 * @return
	 * @throws Exception
	 */
	@GetMapping(value = "/api/data/addition.nhn")
	@ResponseBody
	public Object ifno() throws Exception {

		String additionApiUrl = "https://alpha-oc.bugs.co.kr/bugs/hc/openapi/v1/addition.json";

		String content = "예약번호: 1234567";
		long timestamp = Calendar.getInstance().getTimeInMillis();
		FormBody.Builder formBuilder = new FormBody.Builder();

		// Add parameter
		additionApiUrl += "?content=" + content + "&visible=1";

		Map<String, String> treeMap = new TreeMap<>();
		treeMap.put("content", content);
		treeMap.put("visible", "1");

		String organizationId = "1OW83R3wyBl2FpcZ";
		String uri = "/bugs/hc/openapi/v1/addition.json";
		StringBuilder sb = new StringBuilder(organizationId);
		sb.append(uri);

		for (Map.Entry<String, String> entry : treeMap.entrySet()) {
			sb.append(entry.getValue()).append("&");
		}
		if (sb.lastIndexOf("&") > 0) {
			sb.deleteCharAt(sb.length() - 1); // must delete the last & character!!
		}
		sb.append(timestamp);

		logger.debug("content: " + sb.toString());

		String signString = sign(sb.toString());
		Request request = new Request.Builder().url(additionApiUrl).post(formBuilder.build())
				.header("Content-Type", "application/json;charset=UTF-8").header("Authorization", signString)
				.header("X-TC-Type", "").header("X-TC-Timestamp", Long.toString(timestamp)).build();

		Call call = okHttpClient.newCall(request);
		Response response;
		Map<String, String> map = new HashMap<>();
		try {
			response = call.execute();
			if (response.isSuccessful()) {
				logger.debug("Call addition api successful.");
				String res = response.body().string();
				logger.debug(res);
				map.put("result", res);
				return res;
			} else {
				logger.debug("Call addition api failed.");
				String res = response.body().string();
				logger.debug(res);
				map.put("error", res);
				return res;
			}
		} catch (IOException e) {
			e.printStackTrace();
			logger.debug("Exception: Call addition api failed. " + e.getMessage());
			map.put("error", e.getLocalizedMessage());
		}
		return map;
	}

	private String sign(String content) {
		try {
			logger.info("Content: " + content);
			SecretKeySpec signingKey = new SecretKeySpec(API_KEY.getBytes("UTF-8"), "HmacSHA256");
			Mac mac = Mac.getInstance(signingKey.getAlgorithm());
			mac.init(signingKey);
			byte[] rawHmac = mac.doFinal(content.getBytes("UTF-8"));
			String localAuthorization = new String(Base64.encodeBase64(rawHmac));
			System.out.println(localAuthorization);
			return localAuthorization;
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("content: " + content);
			System.out.println("sign() error: " + e.getMessage());
		}
		return StringUtils.EMPTY;
	}

	public Boolean checkOpenAPIToken(String apiKey, String content, String token) throws Exception {
		SecretKeySpec signingKey = new SecretKeySpec(API_KEY.getBytes("UTF-8"), "HmacSHA256");
		Mac mac = Mac.getInstance(signingKey.getAlgorithm());
		mac.init(signingKey);
		byte[] rawHmac = mac.doFinal(content.getBytes("UTF-8"));
		String localAuthorization = new String(Base64.encodeBase64(rawHmac));
		if (StringUtils.equals(token, localAuthorization)) {
			return true;
		}
		return false;
	}
}
