package com.nhnst.oc.sso.controller;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import com.nhnst.oc.sso.model.User;

import okhttp3.Call;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

/**
 * SSO API remote.json login
 * 
 * @author nhn
 *
 */
@Controller
public class ApiRemoteController extends AbstractLoginController {

	private final Log logger = LogFactory.getLog(ApiRemoteController.class);

	private OkHttpClient client = new OkHttpClient.Builder().followSslRedirects(false).readTimeout(60, TimeUnit.SECONDS)
			.connectTimeout(10, TimeUnit.SECONDS).build();

	@RequestMapping(value = "/api/remote/login.nhn", method = RequestMethod.GET)
	public ModelAndView login(@RequestParam(defaultValue = "") String returnUrl,
			@RequestParam(defaultValue = "") String service, HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		User user = getUserFromCookie(request);
		if (user != null) {
			Long time = postSsoAPI(user, returnUrl, service);
			if (time != null) {
				StringBuilder url = new StringBuilder(returnUrl);
				if (StringUtils.contains(returnUrl, "?")) {
					url.append("&usercode=").append(user.getUserCode());
				} else {
					url.append("?usercode=").append(user.getUserCode());
				}
				url.append("&time=").append(time);
				return new ModelAndView("redirect:" + url.toString());
			}
		}

		Map<String, String> userInfo = new HashMap<String, String>();
		userInfo.put("domain", getDomain());
		userInfo.put("returnUrl", returnUrl);
		return new ModelAndView("login", userInfo);
	}

	@ResponseBody
	@RequestMapping(value = "/remote/login.nhn", method = RequestMethod.GET)
	public String submitLogin(@RequestParam String usercode, @RequestParam String service,
			@RequestParam(defaultValue = "") String username, @RequestParam(defaultValue = "") String email,
			@RequestParam(defaultValue = "") String phone, @RequestParam(defaultValue = "") String returnUrl,
			HttpServletRequest request, HttpServletResponse response) throws Exception {
		// check user login
		User user = new User();
		user.setUserCode(usercode);
		user.setUserName(username);
//		user.setEmail(email);
//		user.setPhone(phone);
		createUserCookie(user, response);

		String time = postSsoAPIStr(user, returnUrl, service);
		if (time != null) {
//			StringBuilder url = new StringBuilder(returnUrl);
//			if (StringUtils.contains(returnUrl, "?")) {
//				url.append("&usercode=").append(user.getUserCode());
//			} else {
//				url.append("?usercode=").append(user.getUserCode());
//			}
//			url.append("&time=").append(time);
			return time;
		}

		return null;
	}

	private Long postSsoAPI(User user, String returnUrl, String serviceId) throws Exception {
		FormBody.Builder formBuilder = new FormBody.Builder();
		formBuilder.add("service", serviceId);
		formBuilder.add("usercode", user.getUserCode());
		formBuilder.add("username", user.getUserName());
//		formBuilder.add("email", user.getEmail());
//		formBuilder.add("phone", user.getPhone());
//		formBuilder.add("memberno", user.getMemberno());
		formBuilder.add("returnUrl", returnUrl);
		Long time = new Date().getTime();
		formBuilder.add("time", time.toString());
		logger.debug(time.toString());
//		String sign = getSHA256Token("0983e74b682b416684d2da59347aec82", serviceId, user, returnUrl, time);
		String sign = getSHA256Token("784b5230e6f641b68e2a4cdbaa679bd7", serviceId, user, returnUrl, time);
		
		logger.debug("token: " + sign);
		formBuilder.add("token", sign);
		StringBuilder url = new StringBuilder();
//		url.append("https://game-platform.alpha-oc.toast.com/v2/enduser/remote.json");
		url.append("http://nhn-cs.dev-oc.toast.com:9003/v2/enduser/remote.json");
		logger.debug("url: " + url.toString());
		Request.Builder builder = new Request.Builder().url(url.toString()).post(formBuilder.build());
		Request req = builder.build();
		Call call = client.newCall(req);
		Response response = call.execute();
		if (response.isSuccessful()) {
			logger.info("success.");
			logger.info("[oc api] response code = " + response.code() + ", body = " + response.body().string());
			return time;
		} else {
			logger.error("[oc api] response code = " + response.code() + ", body = " + response.body().string());
			return null;
		}
	}

	private String postSsoAPIStr(User user, String returnUrl, String serviceId) throws Exception {
		FormBody.Builder formBuilder = new FormBody.Builder();
		formBuilder.add("service", serviceId);
		formBuilder.add("usercode", user.getUserCode());
		formBuilder.add("username", user.getUserName());
//		formBuilder.add("email", user.getEmail());
//		formBuilder.add("phone", user.getPhone());
//		formBuilder.add("memberno", user.getMemberno());
		formBuilder.add("returnUrl", returnUrl);
		Long time = new Date().getTime();
		formBuilder.add("time", time.toString());
		logger.debug(time.toString());
//		String sign = getSHA256Token("0983e74b682b416684d2da59347aec82", serviceId, user, returnUrl, time);
		String sign = getSHA256Token("784b5230e6f641b68e2a4cdbaa679bd7", serviceId, user, returnUrl, time);
		
		logger.debug("token: " + sign);
		formBuilder.add("token", sign);
		StringBuilder url = new StringBuilder();
//		url.append("https://game-platform.alpha-oc.toast.com/v2/enduser/remote.json");
		url.append("http://nhn-cs.dev-oc.toast.com:9003/v2/enduser/remote.json");
		logger.debug("url: " + url.toString());
		Request.Builder builder = new Request.Builder().url(url.toString()).post(formBuilder.build());
		Request req = builder.build();
		Call call = client.newCall(req);
		Response response = call.execute();
		if (response.isSuccessful()) {
			logger.info("success.");
			String result = response.body().string();
			logger.info("[oc api] response code = " + response.code() + ", body = " + result);
			return result;
		} else {
			logger.error("[oc api] response code = " + response.code() + ", body = " + response.body().string());
			return null;
		}
	}
}
